"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeepClient = exports.GLOBAL_ID_REJECTED = exports.GLOBAL_ID_RESOLVED = exports.GLOBAL_ID_THEN = exports.GLOBAL_ID_PROMISE = exports.GLOBAL_ID_ANY = exports.GLOBAL_ID_CONTAIN = exports.GLOBAL_ID_PACKAGE_VERSION = exports.GLOBAL_ID_PACKAGE_ACTIVE = exports.GLOBAL_ID_PACKAGE = exports.DENIED_IDS = exports.ALLOWED_IDS = void 0;
const gql_1 = require("./gql");
const minilinks_1 = require("./minilinks");
const promise_1 = require("./promise");
const reserve_1 = require("./reserve");
exports.ALLOWED_IDS = [5];
exports.DENIED_IDS = [0, 10, 11, 12, 13];
exports.GLOBAL_ID_PACKAGE = 2;
exports.GLOBAL_ID_PACKAGE_ACTIVE = 44;
exports.GLOBAL_ID_PACKAGE_VERSION = 45;
exports.GLOBAL_ID_CONTAIN = 3;
exports.GLOBAL_ID_ANY = 8;
exports.GLOBAL_ID_PROMISE = 9;
exports.GLOBAL_ID_THEN = 10;
exports.GLOBAL_ID_RESOLVED = 11;
exports.GLOBAL_ID_REJECTED = 12;
class DeepClient {
    constructor(options) {
        this._serialize = {
            link: {
                value: 'value',
                relations: {
                    string: 'value',
                    number: 'value',
                    object: 'value',
                    to: 'link',
                    from: 'link',
                    in: 'link',
                    out: 'link',
                    type: 'link',
                    typed: 'link',
                },
            },
            value: {
                relations: {
                    link: 'link',
                },
            },
        };
        this.minilinks = options.minilinks || (0, minilinks_1.minilinks)([]);
        this.apolloClient = options.apolloClient;
        this.table = options.table || 'links';
        this.selectReturning = options.selectReturning || 'id type_id from_id to_id value';
        this.insertReturning = options.insertReturning || 'id';
        this.updateReturning = options.updateReturning || 'id';
        this.deleteReturning = options.deleteReturning || 'id';
        this.defaultSelectName = options.defaultSelectName || 'SELECT';
        this.defaultInsertName = options.defaultInsertName || 'INSERT';
        this.defaultUpdateName = options.defaultUpdateName || 'UPDATE';
        this.defaultDeleteName = options.defaultDeleteName || 'DELETE';
    }
    select(exp, options) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const where = typeof (exp) === 'object' ? Object.prototype.toString.call(exp) === '[object Array]' ? { id: { _in: exp } } : this.serialize(exp, (options === null || options === void 0 ? void 0 : options.table) === this.table || !(options === null || options === void 0 ? void 0 : options.table) ? 'link' : 'value') : { id: { _eq: exp } };
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (options === null || options === void 0 ? void 0 : options.returning) || this.selectReturning;
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultSelectName;
            const q = yield this.apolloClient.query((0, gql_1.generateQuery)({
                queries: [
                    (0, gql_1.generateQueryData)({
                        tableName: table,
                        returning,
                        variables: Object.assign(Object.assign({}, variables), { where })
                    }),
                ],
                name: name,
            }));
            return Object.assign(Object.assign({}, q), { data: (_b = (_a = (q)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.q0 });
        });
    }
    ;
    insert(objects = {}, options) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const _objects = Object.prototype.toString.call(objects) === '[object Array]' ? objects : [objects];
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (options === null || options === void 0 ? void 0 : options.returning) || this.insertReturning;
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultInsertName;
            const q = yield this.apolloClient.mutate((0, gql_1.generateSerial)({
                actions: [(0, gql_1.insertMutation)(table, Object.assign(Object.assign({}, variables), { objects: objects }), { tableName: table, operation: 'insert', returning })],
                name: name,
            }));
            return Object.assign(Object.assign({}, q), { data: (_c = (_b = (_a = (q)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.m0) === null || _c === void 0 ? void 0 : _c.returning });
        });
    }
    ;
    update(exp, value, options) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const where = typeof (exp) === 'object' ? Object.prototype.toString.call(exp) === '[object Array]' ? { id: { _in: exp } } : this.serialize(exp, (options === null || options === void 0 ? void 0 : options.table) === this.table || !(options === null || options === void 0 ? void 0 : options.table) ? 'link' : 'value') : { id: { _eq: exp } };
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (options === null || options === void 0 ? void 0 : options.returning) || this.updateReturning;
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultUpdateName;
            const q = yield this.apolloClient.mutate((0, gql_1.generateSerial)({
                actions: [(0, gql_1.updateMutation)(table, Object.assign(Object.assign({}, variables), { where: exp, _set: value }), { tableName: table, operation: 'update', returning })],
                name: name,
            }));
            return Object.assign(Object.assign({}, q), { data: (_c = (_b = (_a = (q)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.m0) === null || _c === void 0 ? void 0 : _c.returning });
        });
    }
    ;
    delete(exp, options) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const where = typeof (exp) === 'object' ? Object.prototype.toString.call(exp) === '[object Array]' ? { id: { _in: exp } } : this.serialize(exp, (options === null || options === void 0 ? void 0 : options.table) === this.table || !(options === null || options === void 0 ? void 0 : options.table) ? 'link' : 'value') : { id: { _eq: exp } };
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (options === null || options === void 0 ? void 0 : options.returning) || this.deleteReturning;
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultDeleteName;
            const q = yield this.apolloClient.mutate((0, gql_1.generateSerial)({
                actions: [(0, gql_1.deleteMutation)(table, Object.assign(Object.assign({}, variables), { where: exp, returning }), { tableName: table, operation: 'delete', returning })],
                name: name,
            }));
            return Object.assign(Object.assign({}, q), { data: (_c = (_b = (_a = (q)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.m0) === null || _c === void 0 ? void 0 : _c.returning });
        });
    }
    ;
    reserve(count) {
        return (0, reserve_1.reserve)({ count, client: this.apolloClient });
    }
    ;
    await(id) {
        return __awaiter(this, void 0, void 0, function* () {
            return (0, promise_1.awaitPromise)({
                id, client: this.apolloClient,
                Then: yield this.id('@deep-foundation/core', 'Then'),
                Promise: yield this.id('@deep-foundation/core', 'Promise'),
                Resolved: yield this.id('@deep-foundation/core', 'Resolved'),
                Rejected: yield this.id('@deep-foundation/core', 'Rejected'),
            });
        });
    }
    ;
    serialize(exp, env = 'link') {
        var _a, _b, _c, _d, _e, _f;
        if (Object.prototype.toString.call(exp) === '[object Array]')
            return exp.map(this.serialize);
        else if (typeof (exp) === 'object') {
            const keys = Object.keys(exp);
            const result = {};
            for (let k = 0; k < keys.length; k++) {
                const key = keys[k];
                const type = typeof (exp[key]);
                let setted = false;
                if (env === 'link') {
                    if (type === 'string' || type === 'number') {
                        if (key === 'value' || key === type) {
                            setted = result[type] = { value: { _eq: exp[key] } };
                        }
                        else {
                            setted = result[key] = { _eq: exp[key] };
                        }
                    }
                }
                else if (env === 'value') {
                    if (type === 'string' || type === 'number') {
                        setted = result[key] = { _eq: exp[key] };
                    }
                }
                if (!setted)
                    result[key] = ((_c = (_b = (_a = this._serialize) === null || _a === void 0 ? void 0 : _a[env]) === null || _b === void 0 ? void 0 : _b.relations) === null || _c === void 0 ? void 0 : _c[key]) ? this.serialize(exp[key], (_f = (_e = (_d = this._serialize) === null || _d === void 0 ? void 0 : _d[env]) === null || _e === void 0 ? void 0 : _e.relations) === null || _f === void 0 ? void 0 : _f[key]) : exp[key];
            }
            return result;
        }
        else
            return exp;
    }
    ;
    id(start, ...path) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const pckg = { type_id: exports.GLOBAL_ID_PACKAGE, value: start };
            let where = pckg;
            for (let p = 0; p < path.length; p++) {
                const item = path[p];
                const nextWhere = { in: { type_id: exports.GLOBAL_ID_CONTAIN, value: item, from: where } };
                where = nextWhere;
            }
            const q = yield this.select(where);
            if (q.error)
                throw q.error;
            return (((_b = (_a = q === null || q === void 0 ? void 0 : q.data) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id) | ((_c = _ids === null || _ids === void 0 ? void 0 : _ids[start]) === null || _c === void 0 ? void 0 : _c[path === null || path === void 0 ? void 0 : path[0]]) | 0);
        });
    }
    ;
}
exports.DeepClient = DeepClient;
const _ids = {
    '@deep-foundation/core': {
        'Contain': exports.GLOBAL_ID_CONTAIN,
        'Package': exports.GLOBAL_ID_PACKAGE,
        'PackageActive': exports.GLOBAL_ID_PACKAGE_ACTIVE,
        'PackageVersion': exports.GLOBAL_ID_PACKAGE_VERSION,
        'Promise': exports.GLOBAL_ID_PROMISE,
        'Then': exports.GLOBAL_ID_THEN,
        'Resolved': exports.GLOBAL_ID_RESOLVED,
        'Rejected': exports.GLOBAL_ID_REJECTED,
    },
};
//# sourceMappingURL=client.js.map