import { DeepClient } from './client';
import { Link, MinilinksResult } from './minilinks';
export interface PackagerPackageIdentifier {
    name: string;
    version?: string;
    uri?: string;
    type?: string;
    options?: any;
}
export interface PackagerPackage {
    package: PackagerPackageIdentifier;
    data: PackagerPackageItem[];
    dependencies?: {
        [id: number]: PackagerPackageIdentifier;
    };
    strict?: boolean;
    errors?: PackagerError[];
}
export interface PackagerPackageItem {
    id: number | string;
    type?: number | string;
    from?: number | string;
    to?: number | string;
    value?: PackagerValue;
    package?: {
        dependencyId: number;
        containValue: string;
    };
    _?: boolean;
    updated?: string[];
}
export interface PackagerValue {
    id?: number | string;
    link_id?: number | string;
    value?: number | string | any;
}
export declare type PackagerError = any;
export interface PackagerImportResult {
    errors?: PackagerError[];
    ids?: number[];
}
export declare type PackagerMutated = {
    [index: number]: boolean;
};
export interface PackagerExportOptions {
    packageLinkId: number;
}
export interface PackagerLink extends Link<any> {
    value?: any;
}
export declare function sort(pckg: PackagerPackage, data: PackagerPackageItem[], errors?: PackagerError[]): {
    sorted: any[];
};
export declare class Packager<L extends Link<any>> {
    pckg: PackagerPackage;
    client: DeepClient<any>;
    constructor(client: DeepClient<L>);
    fetchPackageNamespaceId(name: string): Promise<{
        error: any;
        namespaceId: number;
    }>;
    fetchDependenciedLinkId(pckg: PackagerPackage, dependedLink: PackagerPackageItem): Promise<number>;
    insertItem(items: PackagerPackageItem[], item: PackagerPackageItem, errors: PackagerError[], mutated: PackagerMutated): Promise<void>;
    insertItems(pckg: PackagerPackage, data: PackagerPackageItem[], counter: number, dependedLinks: PackagerPackageItem[], errors?: PackagerError[], mutated?: {
        [index: number]: boolean;
    }): Promise<any>;
    globalizeIds(pckg: PackagerPackage, ids: number[], links: PackagerPackageItem[]): Promise<{
        global: PackagerPackageItem[];
    }>;
    import(pckg: PackagerPackage): Promise<PackagerImportResult>;
    selectLinks(options: PackagerExportOptions): Promise<MinilinksResult<PackagerLink>>;
    deserialize(pckg: PackagerPackage, errors?: PackagerError[]): Promise<{
        data: PackagerPackageItem[];
        errors?: PackagerError[];
        counter: number;
        dependedLinks: PackagerPackageItem[];
    }>;
    serialize(globalLinks: MinilinksResult<PackagerLink>, options: PackagerExportOptions, pckg: PackagerPackage): Promise<void>;
    export(options: PackagerExportOptions): Promise<PackagerPackage>;
}
