"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Packager = exports.sort = void 0;
const debug_1 = __importDefault(require("debug"));
const minilinks_1 = require("./minilinks");
const debug = (0, debug_1.default)('deeplinks:packager');
function sort(pckg, data, errors = []) {
    let sorted = [];
    if (pckg.strict) {
        sorted = data;
    }
    else {
        for (let i = 0; i < data.length; i++) {
            const item = data[i];
            if (item.value && !item.type) {
                const max = sorted.reduce((p, l, index) => l.id == item.id ? index : p, 0);
                sorted.splice((max) + 1, 0, item);
            }
            else if (item.package) {
                sorted.splice(0, 0, item);
            }
            else {
                const _ = item === null || item === void 0 ? void 0 : item._;
                const firstDependIndex = sorted.findIndex((l, index) => {
                    return l.from == item.id || l.to == item.id || ((l.type == item.id) && !l._);
                });
                const maxDependIndex = sorted.reduce((p, l, index) => {
                    return ((l.id == item.type) && !_) || l.id == item.from || l.id === item.to ? index : p;
                }, 0);
                const _index = (!!~firstDependIndex && firstDependIndex < maxDependIndex ? firstDependIndex : maxDependIndex);
                sorted.splice(_index + 1, 0, item);
            }
        }
    }
    return { sorted };
}
exports.sort = sort;
class Packager {
    constructor(client) {
        this.client = client;
        global.packager = this;
    }
    fetchPackageNamespaceId(name) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const q = yield this.client.select({ value: { _eq: name }, }, {
                    table: `strings`,
                    returning: 'id: link_id'
                });
                return { error: false, namespaceId: (_b = (_a = q === null || q === void 0 ? void 0 : q.data) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id };
            }
            catch (error) {
                debug('fetchPackageNamespaceId error');
                return { error, namespaceId: 0 };
            }
            return { error: true, namespaceId: 0 };
        });
    }
    fetchDependenciedLinkId(pckg, dependedLink) {
        var _a, _b, _c, _d, _e, _f, _g;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const packageName = (_c = (_a = pckg === null || pckg === void 0 ? void 0 : pckg.dependencies) === null || _a === void 0 ? void 0 : _a[(_b = dependedLink === null || dependedLink === void 0 ? void 0 : dependedLink.package) === null || _b === void 0 ? void 0 : _b.dependencyId]) === null || _c === void 0 ? void 0 : _c.name;
                const packageTableName = `strings`;
                const q = yield this.client.select({
                    value: (_d = dependedLink === null || dependedLink === void 0 ? void 0 : dependedLink.package) === null || _d === void 0 ? void 0 : _d.containValue,
                    link: { from: { [packageTableName]: { value: packageName } } },
                }, {
                    table: `strings`,
                    returning: 'id link { id: to_id }'
                });
                return ((_g = (_f = (_e = q === null || q === void 0 ? void 0 : q.data) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.link) === null || _g === void 0 ? void 0 : _g.id) || 0;
            }
            catch (error) {
                debug('fetchDependenciedLinkId error');
                console.log(error);
            }
            return 0;
        });
    }
    insertItem(items, item, errors, mutated) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            debug('insertItem', item);
            try {
                if (item.type) {
                    const insert = { id: +item.id, type_id: +item.type, from_id: +item.from || 0, to_id: +item.to || 0 };
                    const linkInsert = yield this.client.insert(insert, { name: 'IMPORT_PACKAGE_LINK' });
                    if (linkInsert === null || linkInsert === void 0 ? void 0 : linkInsert.errors) {
                        errors.push(linkInsert === null || linkInsert === void 0 ? void 0 : linkInsert.errors);
                    }
                }
                debug('insertItem promise', item.id);
                yield this.client.await(+item.id);
                debug('insertItem promise awaited', item.id);
                if (item.value && !item.package) {
                    debug('insertItem value', item);
                    let valueLink;
                    if (!item.type) {
                        valueLink = items.find(i => i.id === item.id && !!i.type);
                        debug('insertItem .value', item, valueLink);
                    }
                    else {
                        valueLink = item;
                    }
                    if (!valueLink) {
                        errors.push(`Link ${JSON.stringify(item)} for value not founded.`);
                    }
                    else {
                        debug('insertItem tables');
                        const type = typeof ((_a = item === null || item === void 0 ? void 0 : item.value) === null || _a === void 0 ? void 0 : _a.value);
                        const valueInsert = yield this.client.insert(Object.assign({ link_id: valueLink.id }, item.value), { table: `${type}s`, name: 'IMPORT_PACKAGE_VALUE' });
                        debug('insertItem valueInsert', valueInsert);
                        if (valueInsert === null || valueInsert === void 0 ? void 0 : valueInsert.errors)
                            errors.push(valueInsert === null || valueInsert === void 0 ? void 0 : valueInsert.errors);
                    }
                }
            }
            catch (error) {
                debug('insertItem error');
                console.log(error);
                errors.push(error);
            }
            return;
        });
    }
    insertItems(pckg, data, counter, dependedLinks, errors = [], mutated = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                for (let i = 0; i < dependedLinks.length; i++) {
                    const item = dependedLinks[i];
                    yield this.insertItem(data, item, errors, mutated);
                    if (errors.length)
                        return { ids: [] };
                }
                for (let i = 0; i < data.length; i++) {
                    const item = data[i];
                    if (!item.package) {
                        yield this.insertItem(data, item, errors, mutated);
                        if (errors.length)
                            return { ids: [] };
                    }
                }
                return;
            }
            catch (error) {
                debug('insertItems error');
                console.log(error);
                errors.push(error);
            }
            return;
        });
    }
    globalizeIds(pckg, ids, links) {
        return __awaiter(this, void 0, void 0, function* () {
            const global = links.map(l => (Object.assign({}, l)));
            let idsIndex = 0;
            for (let l = 0; l < links.length; l++) {
                const item = links[l];
                const oldId = item.id;
                let newId;
                if (item.package) {
                    newId = yield this.fetchDependenciedLinkId(pckg, item);
                    item.id = newId;
                }
                else if (item.type) {
                    newId = ids[idsIndex++];
                }
                if (item.type || item.package) {
                    for (let l = 0; l < links.length; l++) {
                        const link = links[l];
                        if (link.from === oldId) {
                            global[l].from = newId;
                        }
                        if (link.to === oldId) {
                            global[l].to = newId;
                        }
                        if (link.type === oldId && !link._) {
                            global[l].type = newId;
                        }
                        if (link.id === oldId) {
                            global[l].id = newId;
                        }
                    }
                }
            }
            return { global };
        });
    }
    import(pckg) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const errors = [];
            try {
                if (!((_a = pckg === null || pckg === void 0 ? void 0 : pckg.package) === null || _a === void 0 ? void 0 : _a.name))
                    throw new Error(`!pckg?.package?.name`);
                if (!((_b = pckg === null || pckg === void 0 ? void 0 : pckg.package) === null || _b === void 0 ? void 0 : _b.version))
                    throw new Error(`!pckg?.package?.version`);
                const { data, counter, dependedLinks } = yield this.deserialize(pckg, errors);
                if (errors.length)
                    return { errors };
                const { sorted } = sort(pckg, data, errors);
                if (errors.length)
                    return { errors };
                const mutated = {};
                const ids = yield this.client.reserve(counter + 20);
                const { global } = yield this.globalizeIds(pckg, ids, sorted);
                yield this.insertItems(pckg, global, counter, dependedLinks, errors, mutated);
                if (errors.length)
                    return { errors };
                return { ids, errors };
            }
            catch (error) {
                debug('insertItems error');
                errors.push(error);
            }
            return { ids: [], errors };
        });
    }
    selectLinks(options) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const Contain = yield this.client.id('@deep-foundation/core', 'Contain');
            const Package = yield this.client.id('@deep-foundation/core', 'Package');
            const result = yield this.client.select({
                _or: [
                    { id: { _eq: options.packageLinkId } },
                    { type_id: { _eq: Contain }, from: { id: { _eq: options.packageLinkId } } },
                    { in: { type_id: { _eq: Contain }, from: { id: { _eq: options.packageLinkId } } } },
                ]
            }, {
                name: 'LOAD_PACKAGE_LINKS',
                returning: `
          id type_id from_id to_id value
          type {
            id value
            contains: in(where: { type_id: { _eq: ${Contain} }, from: { type_id: { _eq: ${Package} } } }) {
              id value
              package: from {
                id value
              }
            }
          }
          from {
            id
            contains: in(where: { type_id: { _eq: ${Contain} }, from: { type_id: { _eq: ${Package} } } }) {
              id value
              package: from {
                id value
              }
            }
          }
          to {
            id
            contains: in(where: { type_id: { _eq: ${Contain} }, from: { type_id: { _eq: ${Package} } } }) {
              id value
              package: from {
                id value
              }
            }
          }
        `
            });
            return (0, minilinks_1.minilinks)((_a = (result)) === null || _a === void 0 ? void 0 : _a.data);
        });
    }
    deserialize(pckg, errors = []) {
        return __awaiter(this, void 0, void 0, function* () {
            const Contain = yield this.client.id('@deep-foundation/core', 'Contain');
            const Package = yield this.client.id('@deep-foundation/core', 'Package');
            const PackageNamespace = yield this.client.id('@deep-foundation/core', 'PackageNamespace');
            const Active = yield this.client.id('@deep-foundation/core', 'PackageActive');
            const Version = yield this.client.id('@deep-foundation/core', 'PackageVersion');
            const data = pckg.data.map(l => (Object.assign(Object.assign({}, l), { value: l.value ? Object.assign({}, l.value) : undefined })));
            const containsHash = {};
            let counter = 0;
            const dependedLinks = [];
            for (let l = 0; l < data.length; l++) {
                const item = data[l];
                const local = item.id;
                if (containsHash[local])
                    item.id = containsHash[local];
                else
                    containsHash[local] = item.id = ++counter;
                if (item.package)
                    dependedLinks.push(item);
            }
            for (let l = 0; l < data.length; l++) {
                const item = data[l];
                if (item.type)
                    item.type = containsHash[item.type];
                if (item.from)
                    item.from = containsHash[item.from] || 0;
                if (item.to)
                    item.to = containsHash[item.to] || 0;
            }
            const packageId = ++counter;
            data.push({
                id: packageId,
                type: Package,
                value: { value: pckg.package.name },
                _: true,
            });
            const containsArray = Object.keys(containsHash);
            for (let c = 0; c < containsArray.length; c++) {
                const contain = containsArray[c];
                data.push({
                    id: ++counter,
                    type: Contain,
                    from: packageId,
                    to: containsHash[contain],
                    value: { value: contain },
                    _: true,
                });
            }
            const n = yield this.fetchPackageNamespaceId(pckg.package.name);
            if (n.error)
                errors.push(n.error);
            let namespaceId = n.namespaceId;
            if (!namespaceId) {
                namespaceId = ++counter;
                data.push({
                    id: namespaceId,
                    type: PackageNamespace,
                    value: { value: pckg.package.name },
                    _: true,
                });
                data.push({
                    id: ++counter,
                    type: Active,
                    to: packageId,
                    from: namespaceId,
                    _: true,
                });
            }
            data.push({
                id: ++counter,
                type: Version,
                to: packageId,
                from: namespaceId,
                value: { value: pckg.package.version },
                _: true,
            });
            data.push({
                id: ++counter,
                type: Contain,
                from: namespaceId,
                to: packageId,
                _: true,
            });
            return { data, errors, counter, dependedLinks };
        });
    }
    serialize(globalLinks, options, pckg) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _0, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11;
        return __awaiter(this, void 0, void 0, function* () {
            const Contain = yield this.client.id('@deep-foundation/core', 'Contain');
            let counter = 1;
            let dependenciesCounter = 1;
            const dependencies = {};
            const dependencedPackages = {};
            const handledDepLinks = {};
            const containsByTo = {};
            const links = [];
            for (let l = 0; l < globalLinks.links.length; l++) {
                const link = globalLinks.links[l];
                if (!!link.type_id && !globalLinks.byId[link.type_id] && !((link === null || link === void 0 ? void 0 : link.type_id) === Contain && ((_a = link === null || link === void 0 ? void 0 : link.from) === null || _a === void 0 ? void 0 : _a.id) === options.packageLinkId) && !((link === null || link === void 0 ? void 0 : link.id) === options.packageLinkId)) {
                    const name = (_f = (_e = (_d = (_c = (_b = link === null || link === void 0 ? void 0 : link.type) === null || _b === void 0 ? void 0 : _b.contains) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.package) === null || _e === void 0 ? void 0 : _e.value) === null || _f === void 0 ? void 0 : _f.value;
                    const foundedDep = dependencedPackages[name];
                    if (!!name && !foundedDep) {
                        const depPack = { name: name };
                        const index = dependenciesCounter++;
                        dependencies[index] = depPack;
                        dependencedPackages[name] = index;
                    }
                    if (!handledDepLinks[link.type_id]) {
                        const containValue = (_k = (_j = (_h = (_g = link === null || link === void 0 ? void 0 : link.type) === null || _g === void 0 ? void 0 : _g.contains) === null || _h === void 0 ? void 0 : _h[0]) === null || _j === void 0 ? void 0 : _j.value) === null || _k === void 0 ? void 0 : _k.value;
                        if (!containValue)
                            pckg.errors.push(`Link contain from package to ${(_l = link === null || link === void 0 ? void 0 : link.type) === null || _l === void 0 ? void 0 : _l.id} does not have value.`);
                        links.push({
                            id: counter++,
                            package: { dependencyId: dependencedPackages[name], containValue },
                        });
                        handledDepLinks[link.type_id] = true;
                    }
                }
                if (!!link.from_id && !globalLinks.byId[link.from_id]) {
                    const name = (_r = (_q = (_p = (_o = (_m = link === null || link === void 0 ? void 0 : link.from) === null || _m === void 0 ? void 0 : _m.contains) === null || _o === void 0 ? void 0 : _o[0]) === null || _p === void 0 ? void 0 : _p.package) === null || _q === void 0 ? void 0 : _q.value) === null || _r === void 0 ? void 0 : _r.value;
                    const foundedDep = dependencedPackages[name];
                    if (!!name && !foundedDep) {
                        const depPack = { name: name };
                        const index = dependenciesCounter++;
                        dependencies[index] = depPack;
                        dependencedPackages[name] = index;
                    }
                    if (!handledDepLinks[link.from_id]) {
                        const containValue = (_v = (_u = (_t = (_s = link === null || link === void 0 ? void 0 : link.from) === null || _s === void 0 ? void 0 : _s.contains) === null || _t === void 0 ? void 0 : _t[0]) === null || _u === void 0 ? void 0 : _u.value) === null || _v === void 0 ? void 0 : _v.value;
                        if (!containValue)
                            pckg.errors.push(`Link contain from package to ${(_w = link === null || link === void 0 ? void 0 : link.from) === null || _w === void 0 ? void 0 : _w.id} does not have value.`);
                        links.push({
                            id: counter++,
                            package: { dependencyId: dependencedPackages[name], containValue },
                        });
                        handledDepLinks[link.from_id] = true;
                    }
                }
                if (!!link.to_id && !globalLinks.byId[link.to_id]) {
                    const name = (_1 = (_0 = (_z = (_y = (_x = link === null || link === void 0 ? void 0 : link.to) === null || _x === void 0 ? void 0 : _x.contains) === null || _y === void 0 ? void 0 : _y[0]) === null || _z === void 0 ? void 0 : _z.package) === null || _0 === void 0 ? void 0 : _0.value) === null || _1 === void 0 ? void 0 : _1.value;
                    const foundedDep = dependencedPackages[name];
                    if (!!name && !foundedDep) {
                        const depPack = { name: name };
                        const index = dependenciesCounter++;
                        dependencies[index] = depPack;
                        dependencedPackages[name] = index;
                    }
                    if (!handledDepLinks[link.to_id]) {
                        const containValue = (_5 = (_4 = (_3 = (_2 = link === null || link === void 0 ? void 0 : link.to) === null || _2 === void 0 ? void 0 : _2.contains) === null || _3 === void 0 ? void 0 : _3[0]) === null || _4 === void 0 ? void 0 : _4.value) === null || _5 === void 0 ? void 0 : _5.value;
                        if (!containValue)
                            pckg.errors.push(`Link contain from package to ${(_6 = link === null || link === void 0 ? void 0 : link.to) === null || _6 === void 0 ? void 0 : _6.id} does not have value.`);
                        links.push({
                            id: counter++,
                            package: { dependencyId: dependencedPackages[name], containValue },
                        });
                        handledDepLinks[link.to_id] = true;
                    }
                }
            }
            pckg.dependencies = dependencies;
            for (let l = 0; l < globalLinks.links.length; l++) {
                const link = globalLinks.links[l];
                if (link.type_id === Contain) {
                    containsByTo[+link.to_id] = link;
                }
                else if (link.id !== options.packageLinkId) {
                    links.push(link);
                }
            }
            for (let l = 0; l < links.length; l++) {
                const link = links[l];
                link._id = link.id;
                if (!link.package) {
                    link.id = ((_8 = (_7 = containsByTo[+link.id]) === null || _7 === void 0 ? void 0 : _7.value) === null || _8 === void 0 ? void 0 : _8.value) ? containsByTo[+link.id].value.value : counter++;
                    for (let li = 0; li < ((_9 = link === null || link === void 0 ? void 0 : link.out) === null || _9 === void 0 ? void 0 : _9.length); li++) {
                        const _link = link.out[li];
                        _link.from_id = link.id;
                    }
                    for (let li = 0; li < ((_10 = link === null || link === void 0 ? void 0 : link.in) === null || _10 === void 0 ? void 0 : _10.length); li++) {
                        const _link = link.in[li];
                        _link.to_id = link.id;
                    }
                    for (let li = 0; li < ((_11 = link === null || link === void 0 ? void 0 : link.typed) === null || _11 === void 0 ? void 0 : _11.length); li++) {
                        const _link = link.typed[li];
                        _link.type_id = link.id;
                    }
                    let value;
                    if (link === null || link === void 0 ? void 0 : link.value) {
                        const _12 = link.value, { id: __id, link_id: __link_id } = _12, _value = __rest(_12, ["id", "link_id"]);
                        value = _value;
                    }
                    const newLink = {
                        id: link.id,
                        type: link.type_id,
                    };
                    if (link.from)
                        newLink.from = link.from.id;
                    if (link.to)
                        newLink.to = link.to.id;
                    if (link.package)
                        newLink.package = link.package;
                    if (value)
                        newLink.value = value;
                    pckg.data.push(newLink);
                }
                else {
                    const newLink = {
                        id: link.id,
                        package: link.package,
                    };
                    pckg.data.push(newLink);
                }
            }
        });
    }
    export(options) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const globalLinks = yield this.selectLinks(options);
            const Package = yield this.client.id('@deep-foundation/core', 'Package');
            const packageLink = (_a = globalLinks.links) === null || _a === void 0 ? void 0 : _a.find(l => { var _a, _b; return ((_b = (_a = l === null || l === void 0 ? void 0 : l.type) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.value) === Package; });
            const packageName = (_b = packageLink === null || packageLink === void 0 ? void 0 : packageLink.value) === null || _b === void 0 ? void 0 : _b.value;
            const pckg = {
                package: { name: packageName },
                data: [],
                strict: false,
                errors: [],
            };
            yield this.serialize(globalLinks, options, pckg);
            return pckg;
        });
    }
}
exports.Packager = Packager;
//# sourceMappingURL=packager.js.map