"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.api = void 0;
const debug_1 = __importDefault(require("debug"));
const client_1 = require("@deep-foundation/hasura/client");
const api_1 = require("@deep-foundation/hasura/api");
const apollo_boost_1 = require("apollo-boost");
const vm_1 = __importDefault(require("vm"));
const promise_1 = require("../promise");
const client_2 = require("../client");
const global_ids_1 = require("../global-ids");
const SCHEMA = 'public';
const debug = (0, debug_1.default)('deepcase:eh');
exports.api = new api_1.HasuraApi({
    path: process.env.MIGRATIONS_HASURA_PATH,
    ssl: !!+process.env.MIGRATIONS_HASURA_SSL,
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const client = (0, client_1.generateApolloClient)({
    path: `${process.env.MIGRATIONS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.MIGRATIONS_HASURA_SSL,
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const deep = new client_2.DeepClient({
    apolloClient: client,
});
exports.default = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d, _e, _f, _g;
    try {
        const event = (_a = req === null || req === void 0 ? void 0 : req.body) === null || _a === void 0 ? void 0 : _a.event;
        const operation = event === null || event === void 0 ? void 0 : event.op;
        if (operation === 'INSERT' || operation === 'UPDATE' || operation === 'DELETE') {
            const oldRow = (_b = event === null || event === void 0 ? void 0 : event.data) === null || _b === void 0 ? void 0 : _b.old;
            const newRow = (_c = event === null || event === void 0 ? void 0 : event.data) === null || _c === void 0 ? void 0 : _c.new;
            const current = operation === 'DELETE' ? oldRow : newRow;
            const typeId = current.type_id;
            console.log('current', current, typeId);
            try {
                const handleStringResult = yield client.query({ query: (0, apollo_boost_1.gql) `query SELECT_CODE($typeId: bigint) { links(where: {
          type_id: { _eq: ${yield deep.id('@deep-foundation/core', 'SyncTextFile')} },
          # to_id: { _eq: 16 },
          # from_id: { _eq:  }
          in: {
            from_id: { _eq: ${yield deep.id('@deep-foundation/core', 'JSExecutionProvider')} },
            type_id: { _eq: ${yield deep.id('@deep-foundation/core', 'helloWorldHandler')} },
            in: {
              from: {
                type_id: { _eq: $typeId },
              },
              type_id: { _eq: ${yield deep.id('@deep-foundation/core', 'helloWorldInsertHandler')} },
            }
          }
        }) {
          id
          value
        } }`, variables: {
                        typeId,
                    } });
                const code = (_g = (_f = (_e = (_d = handleStringResult === null || handleStringResult === void 0 ? void 0 : handleStringResult.data) === null || _d === void 0 ? void 0 : _d.links) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.value) === null || _g === void 0 ? void 0 : _g.value;
                if (code) {
                    try {
                        vm_1.default.runInNewContext(code, { console, Error, oldRow, newRow });
                    }
                    catch (error) {
                        debug('error', error);
                    }
                }
                if (operation === 'INSERT' && !global_ids_1.DENIED_IDS.includes(current.type_id) && global_ids_1.ALLOWED_IDS.includes(current.type_id)) {
                    debug('resolve', current.id);
                    yield (0, promise_1.resolve)({
                        id: current.id, client,
                        Then: yield deep.id('@deep-foundation/core', 'Then'),
                        Promise: yield deep.id('@deep-foundation/core', 'Promise'),
                        Resolved: yield deep.id('@deep-foundation/core', 'Resolved'),
                        Rejected: yield deep.id('@deep-foundation/core', 'Rejected'),
                    });
                }
                return res.status(200).json({});
            }
            catch (error) {
                debug('error', error);
                if (operation === 'INSERT' && !global_ids_1.DENIED_IDS.includes(current.type_id) && global_ids_1.ALLOWED_IDS.includes(current.type_id)) {
                    debug('reject', current.id);
                    yield (0, promise_1.reject)({
                        id: current.id, client,
                        Then: yield deep.id('@deep-foundation/core', 'Then'),
                        Promise: yield deep.id('@deep-foundation/core', 'Promise'),
                        Resolved: yield deep.id('@deep-foundation/core', 'Resolved'),
                        Rejected: yield deep.id('@deep-foundation/core', 'Rejected'),
                    });
                }
            }
            return res.status(500).json({ error: 'notexplained' });
        }
        return res.status(500).json({ error: 'operation can be only INSERT or UPDATE' });
    }
    catch (error) {
        return res.status(500).json({ error: error.toString() });
    }
});
//# sourceMappingURL=links.js.map